<?php

namespace Ems\portalGestor\Controller;

use Ems\portalGestor\Service\LancamentoService;
use Zeedhi\Framework\Controller\CrudNext;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DTO\Response\Error;

class LancamentoController {

    public function __construct(
        LancamentoService $lancamentoService
    ) {
        $this->lancamentoService = $lancamentoService;
    }

    public function getLotesLancamento(
        Request\Filter $request,
        Response $response
    ) {
        try{
            $row = $request->getParameters();

            $nrOrg      = (int) $row['NRORG'];
            $cdOperador = (string) $row['CDOPERADOR'];
            $nrPeriodo  = (int) $row['NRPERIODOAPURACAO'];

            $lotes = $this->lancamentoService->getLotesLancamento($nrOrg, $cdOperador, $nrPeriodo);

            $response->addDataSet(new DataSet('data', $lotes));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getLancamentosEvento(
        Request\Filter $request,
        Response $response
    ) {
        try{
            $row = $request->getParameters();

            $nrOrg       = (int) $row['NRORG'];
            $nrLote      = (int) $row['NRLOTELANCEVENTO'];
            $nrEstrutura = (int) $row['NRESTRUTURAM'];
            $nrPeriodo   = (int) $row['NRPERIODOAPURACAO'];

            $lancamentos = $this->lancamentoService->
                getLancamentosEvento($nrOrg, $nrLote, $nrEstrutura, $nrPeriodo);

            $response->addDataSet(new DataSet('data', $lancamentos));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTiposMovimento(
        Request\Filter $request,
        Response $response
    ) {
        try{
            $rows = $request->getParameters();

            $nrOrg = (int) $rows['NRORG'];

            $tiposMov = $this->lancamentoService->getTiposMovimento($nrOrg);

            $response->addDataSet(new DataSet('data', $tiposMov));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function insertLoteLancamento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg         = (int) $row['NRORG'];
            $dsLote        = (string) $row['DSLOTELANCEVENTO'];
            $dtCompetencia = (string) $row['DTCOMPETLOTEEVE'];
            $cdOperador    = (string) $row['CDOPERADOR'];
            $nrTipoMov     = (int) $row['NRTIPOMOVIMENT'];
            $dtOcorrencia;

            if (!isset($row['DTOCORRENCIA']) || $row['DTOCORRENCIA'] === '') {
                $dtOcorrencia = $dtCompetencia;
            } else {
                $dtOcorrencia = $row['DTOCORRENCIA'];
            }

            $success = $this->lancamentoService->insertLoteLancamento(
                $nrOrg, $dsLote, $dtOcorrencia, $dtCompetencia,
                $cdOperador, $nrTipoMov
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getEventosLancamento(
        Request\Filter $request,
        Response $response
    ) {
        try{
            $row = $request->getParameters();

            $nrOrg       = (int) $row['NRORG'];
            $nrEstrutura = (int) $row['NRESTRUTURAM'];

            $eventos = $this->lancamentoService->getEventosLancamento($nrOrg, $nrEstrutura);

            $response->addDataSet(new DataSet('data', $eventos));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function editLoteLancamento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg = (int) $row['NRORG'];
            $nrLote = (int) $row['NRLOTELANCEVENTO'];
            $dsLote = (string) $row['DSLOTELANCEVENTO'];
            $dtCompetencia = (string) $row['DTCOMPETLOTEEVE'];
            $cdOperador = (string) $row['CDOPERADOR'];
            $nrTipoMov = (int) $row['NRTIPOMOVIMENT'];
            $dtOcorrencia;

            if (!isset($row['DTOCORRENCIA']) || $row['DTOCORRENCIA'] === '') {
                $dtOcorrencia = $dtCompetencia;
            } else {
                $dtOcorrencia = $row['DTOCORRENCIA'];
            }

            $success = $this->lancamentoService->editLoteLancamento(
                $nrOrg, $nrLote, $dsLote, $dtOcorrencia, $dtCompetencia,
                $cdOperador, $nrTipoMov
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function confirmLoteLancamento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg = (int) $row['NRORG'];
            $nrLote = (int) $row['NRLOTELANCEVENTO'];

            $success = $this->lancamentoService->confirmLoteLancamento(
                $nrOrg, $nrLote
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function editLancamentos(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg = (int) $row['NRORG'];
            $lancamentos = (array) $row['lancamentos'];
            $cdOperador = (string) $row['CDOPERADOR'];

            $success = $this->lancamentoService->editLancamentos(
                $nrOrg, $lancamentos, $cdOperador
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function deleteLoteLancamento(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg = (int) $row['NRORG'];
            $nrLote = (int) $row['NRLOTELANCEVENTO'];

            $success = $this->lancamentoService->deleteLoteLancamento(
                $nrOrg, $nrLote
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
