<?php

namespace Ems\portalGestor\Util;

use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;

abstract class NovoCodigo
{
    public static function geraCodigo($nrorg, $stcodigo, $intervalo = 1, $tamanhoColuna = 10, $numerico = false, $commit = true)
    {
        $novocodigo = self::executaProcedure($stcodigo, $nrorg, $intervalo, $commit);

        $codigo = self::preparaValor($novocodigo, $tamanhoColuna, $numerico);

        $codigo = self::validCode($codigo, $nrorg, $stcodigo);

        return $codigo;
    }

    public static function validCode($code, $nrorg, $nameTable)
    {

        if ($nameTable == 'FPA_ITECALCFOLHA') {
            return $code;
        }

        try {
            $nrorgWhere = '';

            if ($nrorg != 0) {
                $nrorgWhere = "|| ' WHERE NRORG = $nrorg'";
            }

            $sql = "SELECT 'SELECT  NVL(MAX(' || USER_CONS_COLUMNS.COLUMN_NAME || '), 0) + 1 CODE FROM ' || '$nameTable' $nrorgWhere QRY
                        FROM USER_CONS_COLUMNS
                        JOIN USER_TABLES ON USER_TABLES.TABLE_NAME = USER_CONS_COLUMNS.TABLE_NAME
                        WHERE USER_CONS_COLUMNS.CONSTRAINT_NAME = ( SELECT CONSTRAINT_NAME
                                                                    FROM ALL_CONSTRAINTS
                                                                    WHERE TABLE_NAME    = '$nameTable'
                                                                      AND CONSTRAINT_TYPE = 'P'
                                                                      AND OWNER           = 'FOLHA' )
                          AND USER_CONS_COLUMNS.COLUMN_NAME != 'NRORG'";

            $queryMax = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql);
            if (count($queryMax) > 0) {
                $resultMax  = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($queryMax[0]['QRY']);
                if (count($resultMax) > 0) {
                    $codeMax = $resultMax[0]['CODE'];
                    if ($code < $codeMax) {
                        $codeUpdate = $codeMax + 1;
                        $update = "UPDATE NOVOCODIGO SET NRSEQUENCIAL = $codeUpdate WHERE NRORG = $nrorg AND CDCONTADOR = '$nameTable'";
                        \Setforms2\Shared\Registry::getEntityManagerConnection()->exec($update);
                        return $codeMax;
                    }
                }
            }
            return $code;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @static
     * @param $novocodigo
     * @param $tamanhoColuna
     * @param $numerico
     * @return int|string
     */
    protected static function preparaValor($novocodigo, $tamanhoColuna, $numerico)
    {
        /**
         * Valor do "$novoCodigo" vem uma string com "left zero fill" de tamanho 20, eg "00000000000012345678";
         * Por isso o casting para int, e o substr, lembrando de um exemplo onde o tamanho é 10:
         * substr($novocodigo, -1*$tamanhoColuna) => substr("00000000000012345678", -10) => "0012345678"
         * Lembrando que ele vai pegar os 10 ultimos digitos e/ou os 10 digitos menos significativos, que são os
         * 10 digitos a esquerda.
         */
        return $numerico === true ? (int) $novocodigo : substr($novocodigo, -1 * $tamanhoColuna);
    }

    /**
     * @static
     * @param $stcodigo
     * @param $nrorg
     * @param $intervalo
     * @return mixed
     */
    protected static function executaProcedure($stcodigo, $nrorg, $intervalo, $commit = true)
    {
        // @todo usar o nrorg.
        if ($commit) {
            $procedure = new Procedure('NOVO_CODIGO');
        } else {
            $procedure = new Procedure('NOVO_CODIGO_SC');
        }

        $procedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $stcodigo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE', Param::PARAM_INPUT, $intervalo, Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $nrorg, Param::PARAM_TYPE_INT));
        $novocodigo = $procedure->call();
        return $novocodigo['P_SEQUENCIAL'];
    }

    /**
     * Retorna o codigo gerado pela Novocodigo com a quantidade de caracteres do campo.
     *
     * @static
     * @param $campo
     * @param $tamanho
     * @return string
     *
     */
    public static function getCodigoTamanhoCampo($campo, $tamanho)
    {
        $count       = count($campo);
        $inicial     = $count - ($tamanho + 1);
        $campo = substr($campo, $inicial);
        return $campo;
    }

    public static function selectMax($nomeTabela, $campoMax, $pks = array())
    {
        $sql = "SELECT MAX($campoMax) + 1 CODIGO
                  FROM $nomeTabela 
                  WHERE 1 = 1 
                  WHERE_PLACEHOLDER";
        $where = "";
        foreach ($pks as $campo => $valor) {
            $where .= " AND $campo = '$valor' ";
        }

        $sql = str_replace("WHERE_PLACEHOLDER", $where, $sql);

        try {
            $result = \Setforms2\Shared\Registry::getEntityManager()->getConnection()->fetchAll($sql);
            return isset($result[0]["CODIGO"]) ? $result[0]["CODIGO"] : 1;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function geraNovoCodigoMigracao($nomeTabela, $nomeColuna, $nrorg)
    {
        $cdcontador = strtoupper("INTG_{$nomeTabela}_{$nomeColuna}");
        $novocodigo = \Geral\Entities\Novocodigo::findOneByCdcontadorAndNrorg($cdcontador, $nrorg);

        if ($novocodigo instanceof \Geral\Entities\Novocodigo) {
            $result = self::geraCodigo($nrorg, $cdcontador, 1, 10, true);
        } else {
            $result = self::criaNovoCodigo($nrorg, $cdcontador);
        }

        return $result;
    }

    /**
     * @param $nrorg
     * @param $cdcontador
     * @return float
     * @throws \Exception
     */
    public static function criaNovoCodigo($nrorg, $cdcontador)
    {
        try {
            $nrsequencial = "00000000000001000001";

            $novocodigo = new \Geral\Entities\Novocodigo();
            $novocodigo->setCdcontador($cdcontador);
            $novocodigo->setNrorg($nrorg);
            $novocodigo->setNrsequencial($nrsequencial);
            $novocodigo->setDtinclusao(new \DateTime());
            $novocodigo->setDtultatu(new \DateTime());
            $novocodigo->setNrorginclusao(1);
            $novocodigo->setCdoperinclusao(1);
            $novocodigo->setNrorgultatu(1);
            $novocodigo->setCdoperultatu(1);
            $novocodigo->persist();
            \Setforms2\Shared\Registry::getEntityManager()->flush();

            return floatval($nrsequencial);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
