/* Sistemas Integrados TEKNISA
   Planejamento Alimentao Industrial

Nome: 7080_PLASTPS_CONSUMO_REQUISICAO_EXTRA.SQL

Descrio:
Gera requisio extra de matria prima automaticamente, com base no consumo do
crdapio, consumo fixo ou consumo per-capita de determinado servio, gravando
na tabela PREVRETI.
Obs.: O nmero de comensais utilizado no  o do cardpio e sim o parametrizado
      na tela PLA00910, recebido como parmetro na STP (P_NRCOMENSAIS).

P_CDFILIAL    - Cdigo da Filial
P_CDSERVFILI  - Cdigo do Servio que ser gerada a Requisio Extra
P_DTCARDAPIO  - Data da Requisio
P_NRREQUEXTR  - Nmero da Requisio (poder utilizar um j existente, inserido
                   pelo usurio, ou um sequencial gerado pela tela PLA00910)
P_NRCOMENSAIS - Nmero de Comensais (usados no clculo de consumo do cardpio e
                   consumo per-capita)
P_CDOPERADOR  - Cdigo do Operador
P_IDTIPOCONS  - Identificador do dia da semana ('8' se feriado)

Criao:
24/07/2007 Eurico Jos Faria Klein
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE O.NAME = 'CONSUMO_REQUISICAO_EXTRA'  AND
                          TYPE = 'P'                           AND
                          U.UID = O.UID                        AND
                          U.NAME = USER)
DROP PROCEDURE CONSUMO_REQUISICAO_EXTRA
GO

CREATE PROCEDURE CONSUMO_REQUISICAO_EXTRA
   @P_CDFILIAL    VARCHAR(4),
   @P_CDSERVFILI  VARCHAR(5),
   @P_DTCARDAPIO  DATETIME,
   @P_NRREQUEXTR  VARCHAR(10),
   @P_NRCOMENSAIS DOUBLE PRECISION,
   @P_CDOPERADOR  VARCHAR(12),
   @P_IDTIPOCONS  VARCHAR(1)
AS
   /*VARIVEIS*/
   DECLARE @V_NRPRPREVRETI INT;
   DECLARE @V_HRINICATEN   VARCHAR(4);
   DECLARE @V_NRSEQUENCIAL VARCHAR(4);

   /*VARIVEIS PARA RECEBER DADOS DO CURSOR*/
   DECLARE @V_CDPRODUTO    VARCHAR(10);
   DECLARE @V_QTBRUTA      DOUBLE PRECISION;
   DECLARE @V_IDQTDECONS   VARCHAR(1);
   DECLARE @V_CDTIPOPRAT   VARCHAR(2);
   DECLARE @V_CDPRATO      VARCHAR(10);
   DECLARE @V_CDFILAUXPRAT VARCHAR(4);
   DECLARE @V_IDPREVTPPRAT VARCHAR(1);
BEGIN
   SET @V_NRPRPREVRETI = 1;
   DECLARE C_CARDAPIO CURSOR FOR
      SELECT PR.CDPRODUTO,
             SUM(PR.QTBRUTA) AS QTBRUTA,
             PR.IDQTDECONS,
             (CASE SE.IDUTRETIPRAT WHEN 'S' THEN PR.CDTIPOPRAT   ELSE NULL END) AS CDTIPOPRAT,
             (CASE SE.IDUTRETIPRAT WHEN 'S' THEN PR.CDPRATO      ELSE NULL END) AS CDPRATO,
             (CASE SE.IDUTRETIPRAT WHEN 'S' THEN PR.CDFILAUXPRAT ELSE NULL END) AS CDFILAUXPRAT,
             SE.IDUTRETIPRAT AS IDPREVTPPRAT
      FROM(
           SELECT R.CDPRODUTO,
                  R.QTBRUTASE * @P_NRCOMENSAIS AS QTBRUTA,
                  'C' IDQTDECONS,
                  P.CDTIPOPRAT,
                  P.CDPRATO,
                  P.CDFILAUXPRAT
           FROM CARDADIA C, COMPCARD P, RECESERV R
           WHERE (C.CDFILIAL     = @P_CDFILIAL    )
             AND (C.CDSERVFILI   = @P_CDSERVFILI  )
             AND (C.DTCARDAPIO   = @P_DTCARDAPIO  )
             AND (C.CDFILIAL     = P.CDFILIAL    )
             AND (C.NRCARDAPIO   = P.NRCARDAPIO  )
             AND (R.CDPRATO      = P.CDPRATO     )
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND (R.CDFILIAL     = C.CDFILIAL    )
             AND (R.CDSERVFILI   = C.CDSERVFILI  )
          UNION ALL
           SELECT R.CDPRODUTO,
                  R.QTBRUTALO * @P_NRCOMENSAIS AS QTBRUTA,
                  'C' IDQTDECONS,
                  P.CDTIPOPRAT,
                  P.CDPRATO,
                  P.CDFILAUXPRAT
           FROM CARDADIA C, COMPCARD P, RECEFILI R
           WHERE (C.CDFILIAL     = @P_CDFILIAL    )
             AND (C.CDSERVFILI   = @P_CDSERVFILI  )
             AND (C.DTCARDAPIO   = @P_DTCARDAPIO  )
             AND (C.CDFILIAL     = P.CDFILIAL    )
             AND (C.NRCARDAPIO   = P.NRCARDAPIO  )
             AND (R.CDPRATO      = P.CDPRATO     )
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND (R.CDFILIAL     = C.CDFILIAL    )
             AND NOT EXISTS (SELECT RECESERV.CDPRATO
                             FROM RECESERV
                             WHERE (RECESERV.CDFILIAL     = C.CDFILIAL    )
                               AND (RECESERV.CDSERVFILI   = C.CDSERVFILI  )
                               AND (RECESERV.CDPRATO      = P.CDPRATO     )
                               AND (RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT))
          UNION ALL
           SELECT R.CDPRODUTO,
                  R.QTBRUTAPR * @P_NRCOMENSAIS AS QTBRUTA,
                  'C' IDQTDECONS,
                  P.CDTIPOPRAT,
                  P.CDPRATO,
                  P.CDFILAUXPRAT
           FROM CARDADIA C, COMPCARD P, RECEPADR R
           WHERE (C.CDFILIAL     = @P_CDFILIAL    )
             AND (C.CDSERVFILI   = @P_CDSERVFILI  )
             AND (C.DTCARDAPIO   = @P_DTCARDAPIO  )
             AND (C.CDFILIAL     = P.CDFILIAL    )
             AND (C.NRCARDAPIO   = P.NRCARDAPIO  )
             AND (R.CDPRATO      = P.CDPRATO     )
             AND (R.CDFILAUXPRAT = P.CDFILAUXPRAT)
             AND NOT EXISTS (SELECT RECESERV.CDPRATO
                             FROM RECESERV
                             WHERE (RECESERV.CDFILIAL     = C.CDFILIAL    )
                               AND (RECESERV.CDSERVFILI   = C.CDSERVFILI  )
                               AND (RECESERV.CDPRATO      = P.CDPRATO     )
                               AND (RECESERV.CDFILAUXPRAT = P.CDFILAUXPRAT))
             AND NOT EXISTS (SELECT RECEFILI.CDPRATO
                             FROM RECEFILI
                             WHERE (RECEFILI.CDFILIAL     = C.CDFILIAL    )
                               AND (RECEFILI.CDPRATO      = P.CDPRATO     )
                               AND (RECEFILI.CDFILAUXPRAT = P.CDFILAUXPRAT))
          UNION ALL
           SELECT C.CDPRODUTO,
                  (CASE C.IDQTDECONS WHEN 'P' THEN C.QTCONSSERV * @P_NRCOMENSAIS ELSE C.QTCONSSERV END) AS QTBRUTA,
                  C.IDQTDECONS,
                  (CASE S.IDUTRETIPRAT WHEN 'S'
                      THEN (CASE C.IDQTDECONS WHEN 'P' THEN P.CDTPRCONSPER ELSE P.CDTPRCONSFIX END)
                      ELSE NULL END) AS CDTIPOPRAT,
                  (CASE S.IDUTRETIPRAT WHEN 'S'
                      THEN (CASE C.IDQTDECONS WHEN 'P' THEN P.CDPRACONSPER ELSE P.CDPRACONSFIX END)
                      ELSE NULL END) AS CDPRATO,
                  (CASE S.IDUTRETIPRAT WHEN 'S'
                      THEN (CASE C.IDQTDECONS WHEN 'P' THEN P.CDFILCONSPER ELSE P.CDFILCONSFIX END)
                      ELSE NULL END) AS CDFILAUXPRAT
           FROM CONSSERV C, PARAMPLANEJA P, SERVICO S
           WHERE (C.CDFILIAL   = @P_CDFILIAL  )
             AND (C.CDSERVFILI = @P_CDSERVFILI)
             AND (C.IDTIPOCONS = @P_IDTIPOCONS)
             AND (C.CDFILIAL   = P.CDFILIAL  )
             AND (C.CDFILIAL   = S.CDFILIAL  )
             AND (C.CDSERVFILI = S.CDSERVFILI)
             AND (C.DTCONSSERV IN (SELECT MAX(CONSSERV.DTCONSSERV)
                                   FROM CONSSERV
                                   WHERE (CONSSERV.CDFILIAL    = @P_CDFILIAL  )
                                     AND (CONSSERV.CDSERVFILI  = @P_CDSERVFILI)
                                     AND (CONSSERV.DTCONSSERV <= @P_DTCARDAPIO)
                                     AND (CONSSERV.IDTIPOCONS <> '9')))
         ) PR, SERVICO SE
      WHERE (SE.CDFILIAL   = @P_CDFILIAL  )
        AND (SE.CDSERVFILI = @P_CDSERVFILI)
      GROUP BY PR.CDPRODUTO,
               PR.IDQTDECONS,
               (CASE SE.IDUTRETIPRAT WHEN 'S' THEN PR.CDTIPOPRAT   ELSE NULL END),
               (CASE SE.IDUTRETIPRAT WHEN 'S' THEN PR.CDPRATO      ELSE NULL END),
               (CASE SE.IDUTRETIPRAT WHEN 'S' THEN PR.CDFILAUXPRAT ELSE NULL END),
               SE.IDUTRETIPRAT;

   OPEN C_CARDAPIO;

   FETCH NEXT FROM C_CARDAPIO INTO @V_CDPRODUTO , @V_QTBRUTA , @V_IDQTDECONS  ,
                                   @V_CDTIPOPRAT, @V_CDPRATO , @V_CDFILAUXPRAT,
                                   @V_IDPREVTPPRAT;

   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      SELECT @V_HRINICATEN = HRINICATEN
      FROM SERVICO
      WHERE (CDFILIAL   = @P_CDFILIAL)
        AND (CDSERVFILI = @P_CDSERVFILI)
        AND (IDTIPOSERV <> 'A')
        AND ((IDSERVATIV <> 'N') OR (IDSERVATIV IS NULL));

      /*ESTE DELETE EXCLUI TODO ITEM DA REQUISIO QUE ESTEJA PARAMETRIZADO
        DIFERENTE DA CONFIGURAO ATUAL, EVITANDO, NA MESMA REQUISIO, PRODUTOS
        QUE UTILIZAM E QUE NO UTILIZAM RETIRADAS POR PRATO.*/

      DELETE FROM PREVRETI
      WHERE (CDFILIAL      = @P_CDFILIAL            )
        AND (CDSERVFILI    = @P_CDSERVFILI          )
        AND (DTPREVRETI    = @P_DTCARDAPIO          )
        AND (NRREQUEXTR    = @P_NRREQUEXTR          )
        AND (IDPREVTPPRAT <> (SELECT IDUTRETIPRAT
                              FROM SERVICO
                              WHERE SERVICO.CDFILIAL   = PREVRETI.CDFILIAL
                                AND SERVICO.CDSERVFILI = PREVRETI.CDSERVFILI));

      /*A VARIAVEL @V_NRSEQUENCIAL RECEBE O CAMPO NRPRPREVRETI PARA VERIFICAR
        SE H O MESMO PRODUTO NA REQUISICAO E, AO INVS DE INSERIR O PRODUTO
        NOVAMENTE, ATUALIZA-SE O REGISTRO. SE A VARIVEL ESTIVER NULA, SIGNIFICA
        QUE O PRODUTO NO EST NA REQUISIO E APLICA-SE O INSERT NORMALMENTE.
        A EXPRESSAO @V_IDPREVTPPRAT = 'N' FOI UTILIZADA NO IF PARA EVITAR
        A COMPARAO NULL = NULL NO WHERE, NOS CAMPOS CDPRATO, CDTIPOPRAT E
        CDFILAUXPRAT.*/

      IF (@V_IDPREVTPPRAT = 'N') BEGIN
         SELECT @V_NRSEQUENCIAL = MAX(NRPRPREVRETI)
         FROM PREVRETI
         WHERE (CDFILIAL     = @P_CDFILIAL     )
           AND (CDSERVFILI   = @P_CDSERVFILI   )
           AND (DTPREVRETI   = @P_DTCARDAPIO   )
           AND (NRREQUEXTR   = @P_NRREQUEXTR   )
           AND (IDPREVRETI   = @V_IDQTDECONS   )
           AND (CDPRODUTO    = @V_CDPRODUTO    )
           AND (DTLIMITERETI = @P_DTCARDAPIO   )
           AND (HRLIMITERETI = @V_HRINICATEN   )
      END
      ELSE BEGIN
         SELECT @V_NRSEQUENCIAL = MAX(NRPRPREVRETI)
         FROM PREVRETI
         WHERE (CDFILIAL     = @P_CDFILIAL     )
           AND (CDSERVFILI   = @P_CDSERVFILI   )
           AND (DTPREVRETI   = @P_DTCARDAPIO   )
           AND (NRREQUEXTR   = @P_NRREQUEXTR   )
           AND (IDPREVRETI   = @V_IDQTDECONS   )
           AND (CDPRODUTO    = @V_CDPRODUTO    )
           AND (DTLIMITERETI = @P_DTCARDAPIO   )
           AND (HRLIMITERETI = @V_HRINICATEN   )
           AND (CDPRATO      = @V_CDPRATO      )
           AND (CDTIPOPRAT   = @V_CDTIPOPRAT   )
           AND (CDFILAUXPRAT = @V_CDFILAUXPRAT );
      END;

      IF (@V_NRSEQUENCIAL IS NULL) BEGIN
         INSERT INTO PREVRETI
             (CDFILIAL     , CDSERVFILI, DTPREVRETI  , NRREQUEXTR  ,
              IDPREVRETI   , CDPRODUTO , DTLIMITERETI, HRLIMITERETI,
              NRPRPREVRETI , QTPREVRETI, CDTIPOPRAT  , CDPRATO     ,
              CDFILAUXPRAT , CDOPERADOR, IDPREVTPPRAT)
         VALUES
             (@P_CDFILIAL                , @P_CDSERVFILI, @P_DTCARDAPIO , @P_NRREQUEXTR,
              @V_IDQTDECONS              , @V_CDPRODUTO , @P_DTCARDAPIO , @V_HRINICATEN,
              RIGHT(REPLICATE('0',4) + CONVERT(VARCHAR, @V_NRPRPREVRETI, 103), 4),
                                           @V_QTBRUTA   , @V_CDTIPOPRAT , @V_CDPRATO   ,
              @V_CDFILAUXPRAT            , @P_CDOPERADOR, @V_IDPREVTPPRAT);

         SET @V_NRPRPREVRETI = @V_NRPRPREVRETI + 1;
      END
      ELSE BEGIN
         IF (@V_IDPREVTPPRAT = 'N') BEGIN
            UPDATE PREVRETI
            SET QTPREVRETI = @V_QTBRUTA,
                QTPRERETT1 = 0,
                QTPRERETT2 = 0,
                QTPRERETT3 = 0,
                QTPRERETT4 = 0,
                CDSETORCOZ = NULL
            WHERE (CDFILIAL     = @P_CDFILIAL     )
              AND (CDSERVFILI   = @P_CDSERVFILI   )
              AND (DTPREVRETI   = @P_DTCARDAPIO   )
              AND (NRREQUEXTR   = @P_NRREQUEXTR   )
              AND (IDPREVRETI   = @V_IDQTDECONS   )
              AND (CDPRODUTO    = @V_CDPRODUTO    )
              AND (DTLIMITERETI = @P_DTCARDAPIO   )
              AND (HRLIMITERETI = @V_HRINICATEN   )
              AND (NRPRPREVRETI = @V_NRSEQUENCIAL );
         END
         ELSE BEGIN
            UPDATE PREVRETI
            SET QTPREVRETI = @V_QTBRUTA,
                QTPRERETT1 = 0,
                QTPRERETT2 = 0,
                QTPRERETT3 = 0,
                QTPRERETT4 = 0,
                CDSETORCOZ = NULL
            WHERE (CDFILIAL     = @P_CDFILIAL     )
              AND (CDSERVFILI   = @P_CDSERVFILI   )
              AND (DTPREVRETI   = @P_DTCARDAPIO   )
              AND (NRREQUEXTR   = @P_NRREQUEXTR   )
              AND (IDPREVRETI   = @V_IDQTDECONS   )
              AND (CDPRODUTO    = @V_CDPRODUTO    )
              AND (DTLIMITERETI = @P_DTCARDAPIO   )
              AND (HRLIMITERETI = @V_HRINICATEN   )
              AND (NRPRPREVRETI = @V_NRSEQUENCIAL )
              AND (CDPRATO      = @V_CDPRATO      )
              AND (CDTIPOPRAT   = @V_CDTIPOPRAT   )
              AND (CDFILAUXPRAT = @V_CDFILAUXPRAT );
         END;
      END;

      FETCH NEXT FROM C_CARDAPIO INTO @V_CDPRODUTO , @V_QTBRUTA , @V_IDQTDECONS  ,
                                      @V_CDTIPOPRAT, @V_CDPRATO , @V_CDFILAUXPRAT,
                                      @V_IDPREVTPPRAT;
   END;
   CLOSE C_CARDAPIO;
   DEALLOCATE C_CARDAPIO;
END;
GO



-- VERSAO --
EXEC ATUVEROBJ 'CONSUMO_REQUISICAO_EXTRA','4.11.157';
GO

